@@ -16,21 +16,25 @@ def divide_by(number: float, divisor: float) -> float:
 
 def divide_all(divides: List[Tuple[float, float]]) -> None:
     """Do something impossible every day."""
-    try:
-        for number, divisor in divides:
+
+    for number, divisor in divides:
+        console.print(f"dividing {number} by {divisor}")
+        try:
             result = divide_by(number, divisor)
-            console.print(f"{number} divided by {divisor} is {result}")
-    except Exception:
-        console.print_exception(extra_lines=5, show_locals=True)
+        except Exception:
+            console.print_exception(extra_lines=8, show_locals=True)
+        else:
+            console.print(f" = {result}")
 
 
 DIVIDES = [
     (1000, 200),
     (10000, 500),
+    (1, 0),
     (0, 1000000),
     (3.1427, 2),
+    (888, 0),
     (2 ** 32, 2 ** 16),
-    (1, 0),
 ]
 
 divide_all(DIVIDES)
