@@ -7,6 +7,7 @@ from typing import TYPE_CHECKING, NamedTuple, Optional, Tuple
 
 from ._palettes import EIGHT_BIT_PALETTE, STANDARD_PALETTE, WINDOWS_PALETTE
 from .color_triplet import ColorTriplet
+from .repr import Result, rich_repr
 from .terminal_theme import DEFAULT_TERMINAL_THEME
 
 if TYPE_CHECKING:  # pragma: no cover
@@ -25,6 +26,9 @@ class ColorSystem(IntEnum):
     TRUECOLOR = 3
     WINDOWS = 4
 
+    def __repr__(self) -> str:
+        return f"ColorSystem.{self.name}"
+
 
 class ColorType(IntEnum):
     """Type of color stored in Color class."""
@@ -35,6 +39,9 @@ class ColorType(IntEnum):
     TRUECOLOR = 3
     WINDOWS = 4
 
+    def __repr__(self) -> str:
+        return f"ColorType.{self.name}"
+
 
 ANSI_COLOR_NAMES = {
     "black": 0,
@@ -54,6 +61,7 @@ ANSI_COLOR_NAMES = {
     "bright_cyan": 14,
     "bright_white": 15,
     "grey0": 16,
+    "gray0": 16,
     "navy_blue": 17,
     "dark_blue": 18,
     "blue3": 20,
@@ -89,6 +97,7 @@ ANSI_COLOR_NAMES = {
     "blue_violet": 57,
     "orange4": 94,
     "grey37": 59,
+    "gray37": 59,
     "medium_purple4": 60,
     "slate_blue3": 62,
     "royal_blue1": 63,
@@ -121,7 +130,9 @@ ANSI_COLOR_NAMES = {
     "yellow4": 106,
     "wheat4": 101,
     "grey53": 102,
+    "gray53": 102,
     "light_slate_grey": 103,
+    "light_slate_gray": 103,
     "medium_purple": 104,
     "light_slate_blue": 105,
     "dark_olive_green3": 149,
@@ -148,11 +159,13 @@ ANSI_COLOR_NAMES = {
     "light_salmon3": 173,
     "rosy_brown": 138,
     "grey63": 139,
+    "gray63": 139,
     "medium_purple1": 141,
     "gold3": 178,
     "dark_khaki": 143,
     "navajo_white3": 144,
     "grey69": 145,
+    "gray69": 145,
     "light_steel_blue3": 146,
     "light_steel_blue": 147,
     "yellow3": 184,
@@ -182,6 +195,7 @@ ANSI_COLOR_NAMES = {
     "light_goldenrod2": 222,
     "light_yellow3": 187,
     "grey84": 188,
+    "gray84": 188,
     "light_steel_blue1": 189,
     "yellow2": 190,
     "dark_olive_green1": 192,
@@ -216,30 +230,55 @@ ANSI_COLOR_NAMES = {
     "wheat1": 229,
     "cornsilk1": 230,
     "grey100": 231,
+    "gray100": 231,
     "grey3": 232,
+    "gray3": 232,
     "grey7": 233,
+    "gray7": 233,
     "grey11": 234,
+    "gray11": 234,
     "grey15": 235,
+    "gray15": 235,
     "grey19": 236,
+    "gray19": 236,
     "grey23": 237,
+    "gray23": 237,
     "grey27": 238,
+    "gray27": 238,
     "grey30": 239,
+    "gray30": 239,
     "grey35": 240,
+    "gray35": 240,
     "grey39": 241,
+    "gray39": 241,
     "grey42": 242,
+    "gray42": 242,
     "grey46": 243,
+    "gray46": 243,
     "grey50": 244,
+    "gray50": 244,
     "grey54": 245,
+    "gray54": 245,
     "grey58": 246,
+    "gray58": 246,
     "grey62": 247,
+    "gray62": 247,
     "grey66": 248,
+    "gray66": 248,
     "grey70": 249,
+    "gray70": 249,
     "grey74": 250,
+    "gray74": 250,
     "grey78": 251,
+    "gray78": 251,
     "grey82": 252,
+    "gray82": 252,
     "grey85": 253,
+    "gray85": 253,
     "grey89": 254,
+    "gray89": 254,
     "grey93": 255,
+    "gray93": 255,
 }
 
 
@@ -257,6 +296,7 @@ rgb\(([\d\s,]+)\)$
 )
 
 
+@rich_repr
 class Color(NamedTuple):
     """Terminal color definition."""
 
@@ -269,17 +309,10 @@ class Color(NamedTuple):
     triplet: Optional[ColorTriplet] = None
     """A triplet of color components, if an RGB color."""
 
-    def __repr__(self) -> str:
-        return (
-            f"<color {self.name!r} ({self.type.name.lower()})>"
-            if self.number is None
-            else f"<color {self.name!r} {self.number} ({self.type.name.lower()})>"
-        )
-
     def __rich__(self) -> "Text":
         """Dispays the actual color if Rich printed."""
-        from .text import Text
         from .style import Style
+        from .text import Text
 
         return Text.assemble(
             f"<color {self.name!r} ({self.type.name.lower()})",
@@ -287,6 +320,12 @@ class Color(NamedTuple):
             " >",
         )
 
+    def __rich_repr__(self) -> Result:
+        yield self.name
+        yield self.type
+        yield "number", self.number, None
+        yield "triplet", self.triplet, None
+
     @property
     def system(self) -> ColorSystem:
         """Get the native color system for this color."""
@@ -305,7 +344,7 @@ class Color(NamedTuple):
         return self.type == ColorType.DEFAULT
 
     def get_truecolor(
-        self, theme: "TerminalTheme" = None, foreground=True
+        self, theme: Optional["TerminalTheme"] = None, foreground: bool = True
     ) -> ColorTriplet:
         """Get an equivalent color triplet for this color.
 
@@ -471,7 +510,7 @@ class Color(NamedTuple):
     def downgrade(self, system: ColorSystem) -> "Color":
         """Downgrade a color system to a system with fewer colors."""
 
-        if self.type == ColorType.DEFAULT or self.type == system:
+        if self.type in [ColorType.DEFAULT, system]:
             return self
         # Convert to 8-bit color from truecolor color
         if system == ColorSystem.EIGHT_BIT and self.system == ColorSystem.TRUECOLOR:
@@ -550,7 +589,6 @@ if __name__ == "__main__":  # pragma: no cover
     from .console import Console
     from .table import Table
     from .text import Text
-    from . import box
 
     console = Console()
 
@@ -563,6 +601,8 @@ if __name__ == "__main__":  # pragma: no cover
 
     colors = sorted((v, k) for k, v in ANSI_COLOR_NAMES.items())
     for color_number, name in colors:
+        if "grey" in name:
+            continue
         color_cell = Text(" " * 10, style=f"on {name}")
         if color_number < 16:
             table.add_row(color_cell, f"{color_number}", Text(f'"{name}"'))
